import sys
from osgeo import gdal
from PyQt5.QtWidgets import QApplication, QGraphicsView, QGraphicsScene, QGraphicsPixmapItem, QMainWindow
from PyQt5.QtGui import QImage, QPixmap, QPainter, QCursor
from PyQt5.QtCore import Qt, QTimer
import numpy as np

import sys
from osgeo import gdal
from PyQt5.QtWidgets import QApplication, QGraphicsView, QGraphicsScene, QGraphicsPixmapItem, QMainWindow
from PyQt5.QtGui import QImage, QPixmap, QPainter, QCursor
from PyQt5.QtCore import Qt, QTimer

class ImageViewer(QGraphicsView):
    def __init__(self, image_path):
        super().__init__()
        self.setRenderHint(QPainter.Antialiasing)
        self.setRenderHint(QPainter.SmoothPixmapTransform)

        # Load the image using GDAL
        dataset = gdal.Open(image_path)
        if dataset is None:
            raise ValueError(f"Could not open or find the image: {image_path}")

        # Read the image data for each band
        red_band = dataset.GetRasterBand(1).ReadAsArray()
        green_band = dataset.GetRasterBand(2).ReadAsArray()
        blue_band = dataset.GetRasterBand(3).ReadAsArray()

        # Normalize the image data to 0-255 for each band
        min_val = min(red_band.min(), green_band.min(), blue_band.min())
        max_val = max(red_band.max(), green_band.max(), blue_band.max())
        red_band = ((red_band - min_val) / (max_val - min_val) * 255).astype('uint8')
        green_band = ((green_band - min_val) / (max_val - min_val) * 255).astype('uint8')
        blue_band = ((blue_band - min_val) / (max_val - min_val) * 255).astype('uint8')

        # Create RGB image
        self.image_rgb = self.create_rgb_image(red_band, green_band, blue_band)

        # Convert to QImage
        self.image_qt = self.numpy2qimage(self.image_rgb)

        # Create a scene and add the image to the scene
        self.scene = QGraphicsScene()
        self.image_item = QGraphicsPixmapItem(self.image_qt)
        self.scene.addItem(self.image_item)

        # Set the scene to the view
        self.setScene(self.scene)

        # Enable mouse tracking
        self.setMouseTracking(True)

        # Initial position for panning
        self.last_mouse_pos = None

        # Timer to recenter the mouse
        self.recenter_timer = QTimer(self)
        self.recenter_timer.timeout.connect(self.recenter_mouse)
        self.recenter_timer.start(10)  # Check every 10 milliseconds

    def create_rgb_image(self, red_band, green_band, blue_band):
        height, width = red_band.shape
        rgb_image = np.zeros((height, width, 3), dtype=np.uint8)
        rgb_image[:, :, 0] = red_band
        rgb_image[:, :, 1] = green_band
        rgb_image[:, :, 2] = blue_band
        return rgb_image

    def numpy2qimage(self, image):
        height, width, channel = image.shape
        bytes_per_line = 3 * width
        q_image = QImage(image.data, width, height, bytes_per_line, QImage.Format_RGB888)
        return QPixmap.fromImage(q_image)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.last_mouse_pos = event.pos()
        super().mousePressEvent(event)

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.last_mouse_pos = None
        super().mouseReleaseEvent(event)

    def mouseMoveEvent(self, event):
        if self.last_mouse_pos:
            # Calculate the distance moved by the mouse
            delta = event.pos() - self.last_mouse_pos
            self.last_mouse_pos = event.pos()

            # Adjust the scroll bars by the delta
            self.horizontalScrollBar().setValue(self.horizontalScrollBar().value() - delta.x())
            self.verticalScrollBar().setValue(self.verticalScrollBar().value() - delta.y())
        super().mouseMoveEvent(event)
    
    def recenter_mouse(self):
        self.setMouseTracking(False)
        cursor_pos = self.mapFromGlobal(QCursor.pos())
        if cursor_pos.x() <= 0 or cursor_pos.x() >= self.width() or cursor_pos.y() <= 0 or cursor_pos.y() >= self.height():
            # Recenter the cursor to the middle of the view
            QCursor.setPos(self.mapToGlobal(self.rect().center()))
            self.last_mouse_pos = self.mapToGlobal(self.rect().center())
        self.setMouseTracking(True)

class MainWindow(QMainWindow):
    def __init__(self, image_path):
        super().__init__()
        self.setWindowTitle("Image Viewer")
        self.setGeometry(100, 100, 800, 600)
        self.image_viewer = ImageViewer(image_path)
        self.setCentralWidget(self.image_viewer)

if __name__ == '__main__':
    app = QApplication(sys.argv)
    window = MainWindow('U:/Photos/q18067_171_rgb.tif')
    window.show()
    sys.exit(app.exec_())
